% Program written by Leah Band and Markus Owen,
% February 2020.

% Predicts GA concentrations after an exogeneous GA dose, is called by
% Simulatemodel_endogeneousGA.m
% Part of publication, Rizza et al, PNAS, 2021. 

function [GA_data,pos_data,NcellmeriandEZ,length_data,timeend,timeofnextdiv,Ncellmeri,Lm]=Cellbasedmodel_GA_Arab_prodHillfunc(p)


tstepmax=150;  % Number of division events to simulate - appears to be quasisteady after about 100.
tsmallsteps=100; % Number of time steps between division events.

% Start with 2 cells
p.Ncell=2;
p.GAinit=0.1;
GA_initial=p.GAinit*ones(p.Ncell,1); 
length_initial=p.linit*ones(p.Ncell,1);
initial=[GA_initial;length_initial];

% Specify RER along growth zone:
p.position=[0:3500];
p.RER=[p.RERmeri*ones(1,p.merilength+1),p.REREZ*ones(1,p.EZlength),zeros(1,3500-p.merilength-p.EZlength)];

% Vector Lm stores the cell length as cells leave the meristem. Length of
% Lm is the number of cells that have exited the meristem.
p.Lm=[];

% Specify time to next division for the meristem cells:
poscells = (cumsum(length_initial)-0.5*length_initial)';
p.Ncellmeri=nnz([nonzeros(poscells)<=p.merilength]);
for j=1:p.Ncellmeri
    timeofnextdiv(j)=p.Tcmeri+p.cdse*randn(1);
end

% Set up matrices to store data for plotting:
length_data=zeros(tstepmax+1,p.Ncell+tstepmax);
GA_data=zeros(tstepmax+1,p.Ncell+tstepmax);
pos_data=zeros(tstepmax+1,p.Ncell+tstepmax);
GA_data(1,:)=[GA_initial',zeros(1,tstepmax)];
length_data(1,:)=[length_initial',zeros(1,tstepmax)];

% On each step through the loop, we solve the ODEs for [GA, lengths] until
% the time of the next division using ODE45.
tnow=0;
tic
for je=1:tstepmax;
    
    [tdiv,r]=min(timeofnextdiv); % Calculate the time of the next division, tdiv, and which cell will divide, r.
    
    % Introduce 'event' function, which stops integration when a new cell
    % enters the EZ, we then put the cell length into vector Lm, and
    % then continue with integration until we've reached tdiv.
    options = odeset('Events', @moveintoEZ); 
    
    % Simulate ODEs until next division event:
    te=tnow;
    % disp(['Starting division while loop']) % Option of displaying progress.
    % p
    while te<tdiv
        tvec=[te:(tdiv-te)/tsmallsteps:tdiv]; % Create time vector between current time, tnow or te, and time of next division, tdiv.
        [t,Cvector,te,ye,ie]=ode45(@fullmodel_ODEs,tvec,initial,options,p); % Simulate ODEs until a cell enters EZ or reach the time of next division.
        
        GA_te=Cvector(end,1:p.Ncell);
        lengths_te=Cvector(end,p.Ncell+1:2*p.Ncell);
        initial=[GA_te,lengths_te]';
        
        poscells = (cumsum(lengths_te)-0.5*lengths_te)'; % position of cell centres
        p.Ncellmeri=nnz([nonzeros(poscells)<=p.merilength]); % Number of meristem cells

        % If the 'event' occured, put the cell length for the cell leaving the meristem into Lm.
        if length(ie)>0
            %disp(['# events = ',num2str(length(ie))]) % Option of displaying progress.
            b=lengths_te(p.Ncellmeri+1);
            p.Lm=[b,p.Lm];
        end
    end
    
    GA=Cvector(:,1:p.Ncell);
    lengths=Cvector(:,p.Ncell+1:2*p.Ncell);
    tnow=tdiv; % Update the 'current' time.
    
    % Calculate position of the divided cell
    posdividedcell=sum(lengths(end,1:r-1))+0.5*lengths(end,r);
       
    %disp(sprintf('Before division of cell %d; (Ncellmeri, Ncell) = (%d,%d)',r,p.Ncellmeri,p.Ncell))
    %p  % Option of displaying progress.
    
    % Create initial vector for next time interval - assume that after a cell has
    % divided, GA concentrations are equal to original concentration, cell lengths are half the original length. 
    % Provided cell r is still in the meristem, the division occurs and we
    % have an extra cell. If cell r is no longer in the meristem (due to
    % growth) division does not occur.
    if posdividedcell<=p.merilength
        GA_enter=[GA(end,1:r-1),GA(end,r),GA(end,r),GA(end,r+1:p.Ncell)];
        length_enter=[lengths(end,1:r-1),0.5*lengths(end,r),0.5*lengths(end,r),lengths(end,r+1:p.Ncell)];        
        p.Ncell=p.Ncell+1;  % Update number of cells.
    else
        GA_enter=GA(end,:);
        length_enter=lengths(end,:);
    end
    
    initial=[GA_enter,length_enter]';
   
    % Calculate and record data after division event:
    for k=1:p.Ncell
        pos_data(je+1,k)=sum(length_enter(1:k-1))+0.5*length_enter(k);
        length_data(je+1,k)=length_enter(k);
        GA_data(je+1,k)=GA_enter(k);
    end
    
   % Calculate the number of meristem cells and elongation-zone cells:
    p.Ncellmeri=nnz([nonzeros(pos_data(je+1,:))<=p.merilength]); %Note, nonzeros is needed to ignore entries that havent' been specified yet.
    p.NcellmeriandEZ=nnz([nonzeros(pos_data(je+1,:))<=(p.merilength+p.EZlength)]);
    %disp(['After cell division']) % Option of displaying progress.
    % p
    
    % If the new cell is in the elongation zone, put the cell length for the cell leaving the meristem into Lm.
    if length(p.Lm)~=p.Ncell-p.Ncellmeri
      %  disp('New cell in elongation zone?') % Option of displaying progress.
        b=length_enter(p.Ncellmeri+1);
        p.Lm=[b,p.Lm];
    end

    % Specify time of next division for new cells and put them into the 'timeofnextdiv' vector. 
    td1=p.Tcmeri+p.cdse*randn(1);
    td2=p.Tcmeri+p.cdse*randn(1);
    timeofnextdiv=[timeofnextdiv(1:r-1),tnow+td1,tnow+td2, timeofnextdiv(r+1:end)];
    % The timeofnextdiv is only relevant for meristem cells, cells that
    % have entered the EZ are removed from this vector:
    timeofnextdiv=timeofnextdiv(1:p.Ncellmeri);
    
end
toc

timeend=tnow;
NcellmeriandEZ=p.NcellmeriandEZ;
Ncellmeri=p.Ncellmeri;
Lm=p.Lm;
end

function [dCdt]=fullmodel_ODEs(t,C,p)

GA=C(1:p.Ncell);
celllength=C(p.Ncell+1:2*p.Ncell);

poscells = (cumsum(celllength)-0.5*celllength)'; % position of cell centres
RER=interp1(p.position,p.RER,poscells,'pchip'); % RER of each cell

celllength_dot=celllength.*RER';

prod=p.prodQC+p.E*(poscells').^(p.n)./(p.F^(p.n)+poscells'.^(p.n)); % GA synthesis rate for each cell.

% Meristem ODEs:
GA_dot(1:p.Ncellmeri)=p.Vcytprop*(prod(1:p.Ncellmeri)-p.deg*GA(1:p.Ncellmeri))-p.RERmeri*GA(1:p.Ncellmeri);

% Once cells have left the meristem, we augment the meristem ODEs with
% these EZ ODEs:
if p.Ncell>p.Ncellmeri  
    try 
    GA_dot(p.Ncellmeri+1:p.Ncell)=1./(p.Lm'+p.kappa*(celllength(p.Ncellmeri+1:p.Ncell)-p.Lm'))...
    .*(p.Vcytprop*p.Lm'.*(prod(p.Ncellmeri+1:p.Ncell)-p.deg*GA(p.Ncellmeri+1:p.Ncell))...
    -p.kappa*RER(p.Ncellmeri+1:p.Ncell)'.*celllength(p.Ncellmeri+1:p.Ncell).*GA(p.Ncellmeri+1:p.Ncell));
    catch
        disp('Error!')
    end
end


dCdt=[GA_dot';celllength_dot];

end

function [position,isterminal,direction] = moveintoEZ(t,C,p)

celllength=C(p.Ncell+1:2*p.Ncell);

poscells = (cumsum(celllength)-0.5*celllength)';
p.Ncellmeri=nnz([nonzeros(poscells)<=p.merilength]);


position = (p.Ncell-p.Ncellmeri)-(length(p.Lm)+1); % The value that we want to be zero:
                                                   % Number of values of Lm
                                                   % is one less than
                                                   % number of cells that have left
                                                   % the meristem.
isterminal = 1;  % Halt integration 
direction = 0;   % The zero can be approached from either direction
end



