% Program by Leah Band, Sept 2020

% Predicts GA concentrations after an exogeneous GA dose, is called by
% Simulatemodel_dose.m
% Part of publication, Rizza et al, PNAS, 2021. 


function [pos_0min,GA_0min,NcellmeriandEZ_0min,pos_20min,GA_20min,NcellmeriandEZ_20min]=Cellbasedmodel_GA_Arab_prodHillfunc_afterdose(p,GA_data,length_data,timeend,timeofnextdiv,Lm)

p.Lm=Lm;

tstepmax=10;
tsmallsteps=100; % Number of time steps between division events.
cdstd=0.7;  % This is the Standard Error given in Beenster and Baskin

GA_initial=nonzeros(GA_data(end,:))';
length_initial=nonzeros(length_data(end,:))';
initial=[GA_initial';length_initial'];
p.Ncell=length(length_initial);

p.position=[0:3500];
p.RER=[p.RERmeri*ones(1,p.merilength+1),p.REREZ*ones(1,p.EZlength),zeros(1,3500-p.merilength-p.EZlength)];

% Calculate cell positions and number of meristem and EZ cells
poscells = (cumsum(length_initial)-0.5*length_initial)';
p.Ncellmeri=nnz([nonzeros(poscells)<=p.merilength]);
NcellmeriandEZ_0min=nnz([nonzeros(poscells)<=(p.merilength+p.EZlength)]);

% Set up matrices to store data for plotting:
tnow=timeend;
tend=tnow+0.33; % Predict the GA concentrations 20 min after the dose.

timevec_dose=zeros(tstepmax+1,1);
timevec_dose(1)=tnow;

length_data_dose=zeros(tstepmax+1,p.Ncell+tstepmax);
GA_data_dose=zeros(tstepmax+1,p.Ncell+tstepmax);
pos_data_dose=zeros(tstepmax+1,p.Ncell+tstepmax);

% Initial GA concentrations, lengths and cell positions are specified from
% output of simulations of endogeneous GA.
GA_data_dose(1,:)=[GA_initial,zeros(1,tstepmax)];
length_data_dose(1,:)=[length_initial,zeros(1,tstepmax)];
pos_data_dose(1,:)=[poscells',zeros(1,tstepmax)];

GA_0min=GA_data_dose(1,:);
pos_0min=pos_data_dose(1,:);

% On each step through the loop, we solve the ODEs for [GA, lengths] until
% the time of the next division using ODE45.

tdose=tnow;

[tdiv,r]=min(timeofnextdiv); % Calculate the time of the next division, tdiv, and which cell will divide, r.

je=0;
while tdiv<tdose+1/3   % Simulating until 20 minutes after dose.
    je=je+1;
    
    [tdiv,r]=min(timeofnextdiv); % Calculate the time of the next division, tdiv, and which cell will divide, r.
    
    % Introduce 'event' function, which stops integration when a new cell
    % enters the EZ, we then put the cell length into vector Lm, and
    % then continue with integration until we've reached tdiv.
    options = odeset('Events', @moveintoEZ); 
    
    te=tnow;
    %disp(['Starting division while loop'])
    % p
    while te<tdiv
        tvec=[te:(tdiv-te)/tsmallsteps:tdiv]; % Create time vector between current time, tnow or te, and time of next division, tdiv.
        [t,Cvector,te,ye,ie]=ode45(@fullmodel_ODEs,tvec,initial,options,p);
        GA_te=Cvector(end,1:p.Ncell);
        lengths_te=Cvector(end,p.Ncell+1:2*p.Ncell);
        poscells = (cumsum(lengths_te)-0.5*lengths_te)';
        p.Ncellmeri=nnz([nonzeros(poscells)<=p.merilength]);

        initial=[GA_te,lengths_te]';
        
        % If the 'event' occured, put the cell length for the cell leaving the meristem into Lm.
        if length(ie)>0
            % disp(['# events = ',num2str(length(ie))])
            b=lengths_te(p.Ncellmeri+1);
            p.Lm=[b,p.Lm];
        end
    end
    
    GA=Cvector(:,1:p.Ncell);
    lengths=Cvector(:,p.Ncell+1:2*p.Ncell);
    tnow=tdiv; % Update the 'current' time.
    
    % Calculate position of the divided cell
    posdividedcell=sum(lengths(end,1:r-1))+0.5*lengths(end,r);
    % disp(sprintf('Before division of cell %d; (Ncellmeri, Ncell) = (%d,%d)',r,p.Ncellmeri,p.Ncell))
    % p
    % Create initial vector for next time interval - assume that after a cell has
    % divided, GA concentrations are equal to original concentration, cell lengths are half the original length. 
    % Provided cell r is still in the meristem, the division occurs and we
    % have an extra cell. If cell r is no longer in the meristem (due to
    % growth) division does not occur.
    if posdividedcell<=p.merilength
        GA_enter=[GA(end,1:r-1),GA(end,r),GA(end,r),GA(end,r+1:p.Ncell)];
        length_enter=[lengths(end,1:r-1),0.5*lengths(end,r),0.5*lengths(end,r),lengths(end,r+1:p.Ncell)];        
        p.Ncell=p.Ncell+1;  % Update number of cells.
    else
        GA_enter=GA(end,:);
        length_enter=lengths(end,:);
    end
    
    initial=[GA_enter,length_enter]';
   
    % Calculate and record data after division event:
    for k=1:p.Ncell
        pos_data_dose(je+1,k)=sum(length_enter(1:k-1))+0.5*length_enter(k);
        length_data_dose(je+1,k)=length_enter(k);
        GA_data_dose(je+1,k)=GA_enter(k);
    end
    
   % Calculate the number of meristem cells:
    p.Ncellmeri=nnz([nonzeros(pos_data_dose(je+1,:))<=p.merilength]); %Note, nonzeros is needed to ignore entries that havent' been specified yet.
    p.NcellmeriandEZ=nnz([nonzeros(pos_data_dose(je+1,:))<=(p.merilength+p.EZlength)]);

    %disp(['After cell division'])
    % p
    
    % If the new cell is in the elongation zone, put the cell length for the cell leaving the meristem into Lm.
    if length(p.Lm)~=p.Ncell-p.Ncellmeri
        %disp('New cell in elongation zone?')
        b=length_enter(p.Ncellmeri+1);
        p.Lm=[b,p.Lm];
    end

    % Specify time of next division for new cells and put them into the 'timeofnextdiv' vector. 
    td1=p.Tcmeri+cdstd*randn(1);
    td2=p.Tcmeri+cdstd*randn(1);
    timeofnextdiv=[timeofnextdiv(1:r-1),tnow+td1,tnow+td2, timeofnextdiv(r+1:end)];
    % The timeofnextdiv is only relevant for meristem cells, cells that
    % have entered the EZ are removed from this vector:
    timeofnextdiv=timeofnextdiv(1:p.Ncellmeri);    
    timevec_dose(je+1)=tdiv; % Record time at which division occured
    
end

% The while loop does one too many, so go back to the
% values after the penultimate division:
p.Ncell=p.Ncell-1;
p.Ncellmeri=p.Ncellmeri-1;
if je>0
    te=timevec_dose(je);
    initial=[GA_data_dose(je,1:p.Ncell),length_data_dose(je,1:p.Ncell)]';
else 
    te=timevec_dose(1);
    initial=[GA_data_dose(1,1:p.Ncell),length_data_dose(1,1:p.Ncell)]';
end


% Introduce 'event' function, which stops integration when a new cell
% enters the EZ, we then put the cell length into vector Lm, and
% then continue with integration until we've reached tdiv.
options = odeset('Events', @moveintoEZ);

% disp(['Starting while loop until reach 20 min'])
% p
while te<tend
    tvec=[te:(tend-te)/tsmallsteps:tend]; % Create time vector between current time, tnow or te, and time of next division, tdiv.
    [t,Cvector,te,ye,ie]=ode45(@fullmodel_ODEs,tvec,initial,options,p);
    GA_te=Cvector(end,1:p.Ncell);
    lengths_te=Cvector(end,p.Ncell+1:2*p.Ncell);
    poscells = (cumsum(lengths_te)-0.5*lengths_te)';
    p.Ncellmeri=nnz([nonzeros(poscells)<=p.merilength]);
    
    initial=[GA_te,lengths_te]';
    
    % If the 'event' occured, put the cell length for the cell leaving the meristem into Lm.
    if length(ie)>0
        disp(['# events = ',num2str(length(ie))])
        b=lengths_te(p.Ncellmeri+1);
        p.Lm=[b,p.Lm];
    end
end


GA_20min=GA_te;
lengths_20min=lengths_te;
pos_20min=poscells;
NcellmeriandEZ_20min=nnz([nonzeros(poscells)<=(p.merilength+p.EZlength)]);

end

function [dCdt]=fullmodel_ODEs(t,C,p)

GA=C(1:p.Ncell);
celllength=C(p.Ncell+1:2*p.Ncell);
poscells = (cumsum(celllength)-0.5*celllength)';
RER=interp1(p.position,p.RER,poscells,'pchip');

celllength_dot=celllength.*RER';

prod=p.prodQC+p.E*(poscells').^(p.n)./(p.F^(p.n)+poscells'.^(p.n));

GA_dot(1:p.Ncellmeri)=p.dose*(p.PQC+p.zeta*(poscells(1:p.Ncellmeri)').^p.nd./(p.lambda.^p.nd+poscells(1:p.Ncellmeri)'.^p.nd))...
    +p.Vcytprop*(prod(1:p.Ncellmeri)-p.deg*GA(1:p.Ncellmeri))-p.RERmeri*GA(1:p.Ncellmeri);

% Once cells have left the meristem, we augment the ODEs with these EZ
% equations:
if p.Ncell>p.Ncellmeri  
    try 
    GA_dot(p.Ncellmeri+1:p.Ncell)=p.dose*(p.PQC+p.zeta*(poscells(p.Ncellmeri+1:p.Ncell)').^p.nd./(p.lambda.^p.nd+poscells(p.Ncellmeri+1:p.Ncell)'.^p.nd))...
    +1./(p.Lm'+p.kappa*(celllength(p.Ncellmeri+1:p.Ncell)-p.Lm'))...
    .*(p.Vcytprop*p.Lm'.*(prod(p.Ncellmeri+1:p.Ncell)-p.deg*GA(p.Ncellmeri+1:p.Ncell))...
    -p.kappa*RER(p.Ncellmeri+1:p.Ncell)'.*celllength(p.Ncellmeri+1:p.Ncell).*GA(p.Ncellmeri+1:p.Ncell));
    catch
        disp('Error!')
    end
end


dCdt=[GA_dot';celllength_dot];

end

function [position,isterminal,direction] = moveintoEZ(t,C,p)

celllength=C(p.Ncell+1:2*p.Ncell);

poscells = (cumsum(celllength)-0.5*celllength)';
p.Ncellmeri=nnz([nonzeros(poscells)<=p.merilength]);


position = (p.Ncell-p.Ncellmeri)-(length(p.Lm)+1); % The value that we want to be zero:
                                                   % Number of values of Lm
                                                   % is one less than
                                                   % number of cells that have left
                                                   % the meristem.
isterminal = 1;  % Halt integration 
direction = 0;   % The zero can be approached from either direction
end



