% Program by Leah Band, Sept 2020

% Predicts GA concentrations after an exogeneous GA dose.
% Part of publication, Rizza et al, PNAS, 2021. 
% This program produces Fig 3D,E,F and S8A,B.

function Simulatemodel_dose

% load data:
load experimentaldata.mat

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%55
% Specify parameters:

% External GA concentration (mu M)
p.dose=0.1;

% Cell properties parameters
p.linit=6;
p.rt_to_QC=70;
p.Vcytprop=0.7;
pHvac=5.8;
pHcyt=7.6;
pK=4.2;
p.kappa=(1+10^(pHvac-pK))/(1+10^(pHcyt-pK));
p.cdse=0.7;  % This is the Standard Error given in Beenster and Baskin

% Parameters in the GA to nlsGPS1 relationship
a=1.3;
Bmax=1.7;
h=1.21;
Kd=0.0237;

% Growth parameters
load wt_growthparams_fixedxm.mat
p.RERmeri=RERm;
p.REREZ=REREZ;
p.merilength=merilength;
p.EZlength=EZlength;
p.Tcmeri=log(2)/p.RERmeri; % Must result in exp(RER*Tc)=2, to ensure cells divide once doubled in length.

% Synthesis and degradation rates:
p.n=10;
p.deg=0.01;
p.E=0.0006; % named alpha in paper
p.F=125;   % named xi in paper
p.prodQC=0.00005;

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
%  Simulate dose with constant permeability, Figure 3D.

p.PQC=0.5;
p.zeta=0;
p.lambda=0;
p.nd=2;

Nreps=20;
x_ave=[0:0.1:600];

[GA4_0min_constP_ave,GPS1_0min_constP,GA4_20min_constP_ave,GPS1_20min_constP]=averagereplicates_dose(p,a,Bmax,h,Kd,x_ave,Nreps);

% Plot predicted GPS1 at t=0 and t=20 min, together with the data.
figure
hold on
plot(xposition,emission,'bo')
plot(t20_xposition,t20_emission,'ro')
plot(p.rt_to_QC+x_ave,GPS1_0min_constP,'color',[0 0 0.5],'LineWidth',2)
plot(p.rt_to_QC+x_ave,GPS1_20min_constP,'color',[0.8 0 0],'LineWidth',2)
xlim([0 500])
legend1=legend('data 0 min','data 20 min','model 0 min','model 20 min');
set(legend1,'FontSize',14)
xlabel('Distance from root tip ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel('nlsGPS1 emission ratio','Interpreter','latex','FontSize',14)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
% Simulate GA dose with increasing permeability

p.PQC=0.15;
p.zeta=9;
p.lambda=400;
p.nd=2;

% Plot prescribed relationship between permeability and distance from the
% QC, Fig 3E.
xplotvec=[0:0.1:1100];
figure
plot(xplotvec,p.PQC+p.zeta*(xplotvec).^p.nd./(p.lambda.^p.nd+xplotvec.^p.nd),'LineWidth',2)
xlabel('Distance from QC ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel('Permeability','Interpreter','latex','FontSize',14)

% Simulate GA concentrations and average:
[GA4_0min_satP_ave,GPS1_0min_satP,GA4_20min_satP_ave,GPS1_20min_satP]=averagereplicates_dose(p,a,Bmax,h,Kd,x_ave,Nreps);

% Plot GPS1 at t=0 and t=20 min, Figure 3F
figure
hold on
plot(xposition,emission,'bo')
plot(t20_xposition,t20_emission,'ro')
plot(p.rt_to_QC+x_ave,GPS1_0min_satP,'color',[0 0 0.5],'LineWidth',2)
plot(p.rt_to_QC+x_ave,GPS1_20min_satP,'color',[0.8 0 0],'LineWidth',2)
xlim([0 500])
legend1=legend('data 0 min','data 20 min','model 0 min','model 20 min');
set(legend1,'FontSize',14)
xlabel('Distance from root tip ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel('nlsGPS1 emission ratio','Interpreter','latex','FontSize',14)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulations for GA dose for ga2ox knockout mutant, Figure S8A,B
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5

% Parameters relevant for ga2ox
p.deg=0.1*0.01;

% ga2ox growth parameters
load ga2ox_growthparams_fixedxm.mat % load ga2ox growth parameters
p.RERmeri=RERm;
p.REREZ=REREZ;
p.merilength=merilength;
p.EZlength=EZlength;
p.Tcmeri=log(2)/p.RERmeri;  % Cell cycle duration in hours. % Must result in exp(RER*Tc)=2, to ensure cells divide once doubled in length.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
% Simulate GA dose for ga2ox with constant permeability:

p.PQC=0.5;
p.zeta=0;
p.lambda=0;
p.nd=2;

[GA4_0min_ga2ox_ave,GPS1_0min_ga2ox,GA4_20min_ga2ox_ave,GPS1_20min_ga2ox]=averagereplicates_dose(p,a,Bmax,h,Kd,x_ave,Nreps);

% Plot GPS1 for wild-type and ga2ox at t=0 and t=20 min, Figure S8A.
figure
hold on
plot(x_ave,GPS1_0min_constP,'b','LineWidth',2)
plot(x_ave,GPS1_20min_constP,'r','LineWidth',2)
plot(x_ave(1:200:end),GPS1_0min_ga2ox(1:200:end),'b--*','LineWidth',2)
plot(x_ave(1:200:end),GPS1_20min_ga2ox(1:200:end),'r--*','LineWidth',2)
ylim([1.2 3])
legend1=legend('Col0, t=0 min','Col0, t=20 min','{\it{ga2ox q}}, t=0 min','{\it{ga2ox q}}, t=20 min');
set(legend1,'FontSize',14)
xlabel('Distance from root tip ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel('nlsGPS1 emission ratio','Interpreter','latex','FontSize',14)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
% Simulate GA dose for ga2ox with increasing permeability:

p.PQC=0.15;
p.zeta=9;
p.lambda=400;
p.nd=2;

[GA4_0min_ga2ox_satP_ave,GPS1_0min_ga2ox_satP,GA4_20min_ga2ox_satP_ave,GPS1_20min_ga2ox_satP]=averagereplicates_dose(p,a,Bmax,h,Kd,x_ave,Nreps);

% Plot GPS1 for wild-type and ga2ox at t=0 and t=20 min, Figure S8B.
figure
hold on
plot(x_ave,GPS1_0min_satP,'b','LineWidth',2)
plot(x_ave,GPS1_20min_satP,'r','LineWidth',2)
plot(x_ave(1:200:end),GPS1_0min_ga2ox_satP(1:200:end),'b--*','LineWidth',2)
plot(x_ave(1:200:end),GPS1_20min_ga2ox_satP(1:200:end),'r--*','LineWidth',2)
ylim([1.2 3])
legend1=legend('Col0, t=0 min','Col0, t=20 min','{\it{ga2ox q}}, t=0 min','{\it{ga2ox q}}, t=20 min');
set(legend1,'FontSize',14)
xlabel('Distance from QC ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel('nlsGPS1 emission ratio','Interpreter','latex','FontSize',14)


save doseresults_wildtypeandga2ox_fittedparams.mat

end

function [GA4_0min_ave,GPS1_0min,GA4_20min_ave,GPS1_20min]=averagereplicates_dose(p,a,Bmax,h,Kd,x_ave,Nreps)

GA_0min_reps=zeros(Nreps,6001);
GA_20min_reps=zeros(Nreps,6001);
for i=1:Nreps
    % Simulate steady-state GA distribution before the dose:
    [GA_data_wt,pos_data_wt,NcellmeriandEZ_wt,length_data_wt,timeend_wt,timeofnextdiv_wt,Ncellmeri_wt,Lm_wt]=Cellbasedmodel_GA_Arab_prodHillfunc(p);
    [pos_0min_satP,GA_0min_satP,NcellmeriandEZ_0min,pos_20min_satP,GA_20min_satP,NcellmeriandEZ_20min]=Cellbasedmodel_GA_Arab_prodHillfunc_afterdose(p,GA_data_wt,length_data_wt,timeend_wt,timeofnextdiv_wt,Lm_wt);
    
    GA_0min=GA_0min_satP(1:NcellmeriandEZ_0min);
    pos_0min=pos_0min_satP(1:NcellmeriandEZ_0min);
    GA_20min=GA_20min_satP(1:NcellmeriandEZ_20min);
    pos_20min=pos_20min_satP(1:NcellmeriandEZ_20min);
    
    GA_0min_reps(i,:)=interp1(pos_0min,GA_0min,x_ave,'linear');
    GA_20min_reps(i,:)=interp1(pos_20min,GA_20min,x_ave,'linear');
end

GA4_0min_ave=mean(GA_0min_reps);
GPS1_0min=a+Bmax*GA4_0min_ave.^h./(Kd.^h+GA4_0min_ave.^h);

GA4_20min_ave=mean(GA_20min_reps);
GPS1_20min=a+Bmax*GA4_20min_ave.^h./(Kd.^h+GA4_20min_ave.^h);

end