% Program by Leah Band, Sept 2020

% Predicts endogeneous GA concentrations in root growth zones.
% Part of publication, Rizza et al, PNAS, 2021.
% This program produces Fig 1 F,G, Fig S4, Fig S5E,

function Simulatemodel_endogenousGA

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load experimental data on emission ratios
load experimentaldata.mat

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Specify model parameters, as listed in Table S1

% Growth Parameters 
load wt_growthparams_fixedxm.mat
p.RERmeri=RERm;
p.REREZ=REREZ;
p.merilength=merilength;
p.EZlength=EZlength;
p.Tcmeri=log(2)/p.RERmeri; % Must result in exp(RER*Tc)=2, to ensure cells divide once doubled in length.

% Cell properties parameters
p.linit=6;
p.rt_to_QC=70;
p.Vcytprop=0.7;
pHvac=5.8;
pHcyt=7.6;
pK=4.2;
p.kappa=(1+10^(pHvac-pK))/(1+10^(pHcyt-pK));
p.cdse=0.7;  % This is the Standard Error given in Beenster and Baskin

% Parameters in the GA to nlsGPS1 relationship
a=1.3;
Bmax=1.7;
h=1.21;
Kd=0.0237;

% Synthesis and degradation rates (fitted values)
p.prodQC=0.00005;
p.E=0.0006; % named alpha in paper
p.F=125; % named xi in paper
p.deg=0.01;
p.n=10;

% Plot prescribed relationship between GA synthesis and distance for Fig S4: 
xplotvec=[0:0.1:600];
figure
hold on
plot(xplotvec,p.prodQC+p.E*(xplotvec).^p.n./(p.F.^p.n+xplotvec.^p.n),'LineWidth',2)
xlabel('Distance from QC ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel('GA synthesis rate, $\mu$M/hr','Interpreter','latex','FontSize',14)
 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Simulations of endogeneous GA dynamics for wild type
Nreps=20; % Number of replicates
x_ave=[0:0.1:600]; 

GA_reps=zeros(Nreps,6001);

% Calculate replicates and produce Fig S5E showing replicates.
figure 
hold on
for i=1:20
    [GA_data_wt,pos_data_wt,NcellmeriandEZ_wt]=Cellbasedmodel_GA_Arab_prodHillfunc(p);
    GA4=GA_data_wt(end,1:NcellmeriandEZ_wt);
    pos=pos_data_wt(end,1:NcellmeriandEZ_wt);
    plot(pos, GA4,'LineWidth',2) 
    GA_reps(i,:)=interp1(pos,GA4,x_ave,'linear');
end
xlim([0 500])
xlabel('Distance from QC ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel(' $\mu$M GA4','Interpreter','latex','FontSize',14)

GA4_ave=mean(GA_reps); % Calculate average from replicates
GPS1=a+Bmax*GA4_ave.^h./(Kd.^h+GA4_ave.^h); % Calculate GPS1

% Plot of GPS1 predictions together with emission data, Fig 1F. 
figure
hold on
plot(xposition,emission,'ko')
plot(p.rt_to_QC+x_ave, GPS1,'r','LineWidth',2) 
xlim([0 550])
legend1=legend('Data','Model prediction');
set(legend1,'FontSize',14);
xlabel('Distance from root tip ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel('nlsGPS1 emission ratio','Interpreter','latex','FontSize',14)

% Plot of GA and GPS1 together for SI Fig S4
figure
subplot(1,2,1)
hold on
plot(x_ave, GA4_ave,'LineWidth',2) 
xlim([0 600])
xlabel('Distance from QC ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel(' $\mu$M GA4','Interpreter','latex','FontSize',14)
subplot(1,2,2)
hold on
plot(xposition,emission,'ko')
plot(p.rt_to_QC+x_ave, GPS1,'r','LineWidth',2) 
xlim([0 600])
legend1=legend('Data','Model prediction');
set(legend1,'FontSize',14);
xlabel('Distance from root tip ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel('nlsGPS1 emission ratio','Interpreter','latex','FontSize',14)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
% Simulation for ga20ox ko mutant (assumed synthesis rate 10% of wildtype)

p.prodQC=0.1*0.00005;
p.E=0.1*0.0006;
p.F=125;
p.deg=0.01;
p.n=10;

load ga20ox_growthparams_fixedxm.mat % load ga20ox growth parameters
p.RERmeri=RERm;
p.REREZ=REREZ;
p.merilength=merilength;
p.EZlength=EZlength;
p.Tcmeri=log(2)/p.RERmeri;  % Cell cycle duration in hours. Must result in exp(RER*Tc)=2, to ensure cells divide once doubled in length.

% Simulate GA dynamics and calculate average of the replicates:
[GA4_ga20ox_ave,GPS1_ga20ox]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulation for ga2ox mutant (assumed 10% of wildtype degradation)

p.prodQC=0.00005;
p.E=0.0006;
p.F=125;
p.deg=0.1*0.01;
p.n=10;

load ga2ox_growthparams_fixedxm.mat  % load growth parameters.
p.RERmeri=RERm;
p.REREZ=REREZ;
p.merilength=merilength;
p.EZlength=EZlength;
p.Tcmeri=log(2)/p.RERmeri;  % Cell cycle duration in hours. % Must result in exp(RER*Tc)=2, to ensure cells divide once doubled in length.

% Simulate GA dynamics and calculate average of the replicates:
[GA4_ga2ox_ave,GPS1_ga2ox]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);

% Plot figure showing mutant simulations (paper Fig 1G)
figure
hold on
plot(x_ave, GPS1,'LineWidth',2)
plot(x_ave, GPS1_ga2ox,'g','LineWidth',2)
plot(x_ave, GPS1_ga20ox,'r','LineWidth',2)
xlim([0 600])
legend1=legend('Col0','\it{ga2ox}','\it{ga20ox}');
set(legend1,'FontSize',12);
xlabel('Distance from QC ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel('nlsGPS1 emission ratio','Interpreter','latex','FontSize',14)

% Plot of GA and GPS1 for mutants together for SI Fig S4
figure
subplot(1,2,1)
hold on
plot(x_ave, GA4_ave,'LineWidth',2) 
plot(x_ave, GA4_ga2ox_ave,'g','LineWidth',2)
plot(x_ave, GA4_ga20ox_ave,'r','LineWidth',2)
legend1=legend('Col0','\it{ga2ox}','\it{ga20ox}');
set(legend1,'FontSize',14);
xlim([0 600])
xlabel('Distance from QC ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel(' $\mu$M GA4','Interpreter','latex','FontSize',14)
subplot(1,2,2)
hold on
plot(x_ave, GPS1,'LineWidth',2) 
plot(x_ave, GPS1_ga2ox,'g','LineWidth',2)
plot(x_ave, GPS1_ga20ox,'r','LineWidth',2)
xlim([0 600])
legend1=legend('Col0','\it{ga2ox}','\it{ga20ox}');
set(legend1,'FontSize',14);
xlabel('Distance from root tip ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel('nlsGPS1 emission ratio','Interpreter','latex','FontSize',14)

save simulationdata_endogeneousGA_wtandmutants.mat

end

function [GA4_ave,GPS1]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps)

GA_reps=zeros(Nreps,6001);
for i=1:Nreps
    [GA_data_wt,pos_data_wt,NcellmeriandEZ_wt]=Cellbasedmodel_GA_Arab_prodHillfunc(p);
    GA4=GA_data_wt(end,1:NcellmeriandEZ_wt);
    pos=pos_data_wt(end,1:NcellmeriandEZ_wt);
    GA_reps(i,:)=interp1(pos,GA4,x_ave,'linear');
end

GA4_ave=mean(GA_reps);
GPS1=a+Bmax*GA4_ave.^h./(Kd.^h+GA4_ave.^h);

end

