% Program by Leah Band, Sept 2020

% Predicts endogeneous GA concentrations in root growth zones
% and investigates the affect of parameter choices.
% Part of publication, Rizza et al, PNAS, 2021.
% This program produces Fig 1D,E, and Fig S5 A-D.

function Simulatemodel_endogenousGA_affectofparameters

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load experimental data on emission ratios

load experimentaldata.mat

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Specify model parameters, as listed in Table S1

% Growth Parameters 
load wt_growthparams_fixedxm.mat
p.RERmeri=RERm;
p.REREZ=REREZ;
p.merilength=merilength;
p.EZlength=EZlength;
p.Tcmeri=log(2)/p.RERmeri; % Must result in exp(RER*Tc)=2, to ensure cells divide once doubled in length.

% Cell properties parameters
p.linit=6;
p.rt_to_QC=70;
p.Vcytprop=0.7;
pHvac=5.8;
pHcyt=7.6;
pK=4.2;
p.kappa=(1+10^(pHvac-pK))/(1+10^(pHcyt-pK));
p.cdse=0.7;  % This is the Standard Error given in Beenster and Baskin

% Parameters in the GA to nlsGPS1 relationship
a=1.3;
Bmax=1.7;
h=1.21;
Kd=0.0237;

% Synthesis and degradation rates (fitted values)
p.prodQC=0.00005;
p.E=0.0006; % named alpha in paper
p.F=125; % named xi in paper
p.deg=0.01;
p.n=10;

Nreps=20;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%55
% Plot for different E (called alpha in the paper), fig 1D,E

dx=0.1;
x_ave=[0:dx:600];

figure % Figure 1D.
hold on

p.E=0;
plot(x_ave,p.prodQC+p.E*(x_ave).^p.n./(p.F.^p.n+x_ave.^p.n),'LineWidth',2)
[GA4_ave_E0,GPS1_E0]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);

p.E=0.0002;
plot(x_ave,p.prodQC+p.E*(x_ave).^p.n./(p.F.^p.n+x_ave.^p.n),'LineWidth',2)
[GA4_ave_E0002,GPS1_E0002]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);

p.E=0.0004;
plot(x_ave,p.prodQC+p.E*(x_ave).^p.n./(p.F.^p.n+x_ave.^p.n),'LineWidth',2)
[GA4_ave_E0004,GPS1_E0004]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);

p.E=0.0006;
plot(x_ave,p.prodQC+p.E*(x_ave).^p.n./(p.F.^p.n+x_ave.^p.n),'LineWidth',2)
[GA4_ave_E0006,GPS1_E0006]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);

p.E=0.0008;
plot(x_ave,p.prodQC+p.E*(x_ave).^p.n./(p.F.^p.n+x_ave.^p.n),'LineWidth',2)
[GA4_ave_E0008,GPS1_E0008]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);
legend1=legend('\alpha=0','\alpha=0.0002','\alpha=0.0004','\alpha=0.0006','\alpha=0.0008');
set(legend1,'FontSize',14);
xlabel('Distance from QC ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel('GA synthesis rate, $\sigma$(x), $\mu$M/hr','Interpreter','latex','FontSize',14)

figure % Figure 1E
hold on
plot(x_ave, GA4_ave_E0,'LineWidth',2) 
plot(x_ave, GA4_ave_E0002,'LineWidth',2) 
plot(x_ave, GA4_ave_E0004,'LineWidth',2) 
plot(x_ave, GA4_ave_E0006,'LineWidth',2) 
plot(x_ave, GA4_ave_E0008,'LineWidth',2) 
xlim([0 600])
legend1=legend('No EZ synthesis','\alpha=0','\alpha=0.0002','\alpha=0.0004','\alpha=0.0006','\alpha=0.0008');
xlabel('Distance from QC ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel(' $\mu$M GA4','Interpreter','latex','FontSize',14)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%55

p.E=0.0006;

% Plot for different meristem RER, Fig S5A,

p.RERmeri=0.02;
p.Tcmeri=log(2)/p.RERmeri;
[GA4_ave_RERm002,GPS1_RERm002]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);
p.RERmeri=0.03;
p.Tcmeri=log(2)/p.RERmeri;
[GA4_ave_RERm003,GPS1_RERm003]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);
p.RERmeri=0.04;
p.Tcmeri=log(2)/p.RERmeri;
[GA4_ave_RERm004,GPS1_RERm004]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);

figure
hold on
plot(x_ave, GA4_ave_RERm002,'LineWidth',2) 
plot(x_ave, GA4_ave_RERm003,'LineWidth',2) 
plot(x_ave, GA4_ave_RERm004,'LineWidth',2) 
xlim([0 600])
legend1=legend('$RER_{m}=0.02$','$RER_{m}=0.03$','$RER_{m}=0.04$');%,'$\kappa=0.2$','$\kappa=0.5$','$\kappa=1$');
set(legend1,'Interpreter','latex','FontSize',14);
xlabel('Distance from QC ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel(' $\mu$M GA4','Interpreter','latex','FontSize',14)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%55
% Plot for different EZ RER:p.RERmeri=RERm; Fig S5B,
p.Tcmeri=log(2)/p.RERmeri;

p.REREZ=0.1;
[GA4_ave_REREZ01,GPS1_REREZ01]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);
p.REREZ=0.2;
[GA4_ave_REREZ02,GPS1_REREZ02]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);
p.REREZ=0.3;
[GA4_ave_REREZ03,GPS1_REREZ03]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);

figure
hold on
plot(x_ave, GA4_ave_REREZ01,'LineWidth',2) 
plot(x_ave, GA4_ave_REREZ02,'LineWidth',2) 
plot(x_ave, GA4_ave_REREZ03,'LineWidth',2) 
xlim([0 600])
legend1=legend('$RER_{EZ}=0.1$','$RER_{EZ}=0.2$','$RER_{EZ}=0.3$');
set(legend1,'Interpreter','latex','FontSize',14);
xlabel('Distance from QC ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel(' $\mu$M GA4','Interpreter','latex','FontSize',14)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%55
% Plot for different kappa: Fig S5D

p.REREZ=REREZ;

p.kappa=(1+10^(pHvac-pK))/(1+10^(pHcyt-pK));
[GA4_ave,GPS1]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);
p.kappa=0.05;
[GA4_ave_k005,GPS1_k005]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);
p.kappa=0.1;
[GA4_ave_k01,GPS1_k01]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);
p.kappa=0.2;
[GA4_ave_k02,GPS1_k02]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);
p.kappa=0.5;
[GA4_ave_k05,GPS1_k05]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);
p.kappa=1;
[GA4_ave_k1,GPS1_k1]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);

figure
hold on
plot(x_ave, GA4_ave,'LineWidth',2) 
plot(x_ave, GA4_ave_k005,'LineWidth',2)
plot(x_ave, GA4_ave_k01,'LineWidth',2) 
plot(x_ave, GA4_ave_k02,'LineWidth',2) 
plot(x_ave, GA4_ave_k05,'LineWidth',2) 
plot(x_ave, GA4_ave_k1,'LineWidth',2) 
xlim([0 600])
ylim([0, 0.025])
legend1=legend('$\kappa=0.016$','$\kappa=0.05$','$\kappa=0.1$','$\kappa=0.2$','$\kappa=0.5$','$\kappa=1$');
set(legend1,'Interpreter','latex','FontSize',14);
xlabel('Distance from QC ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel(' $\mu$M GA4','Interpreter','latex','FontSize',14)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot for different proportions of the meristem cell volume being
% cytoplasm: Fig S5C.
p.kappa=(1+10^(pHvac-pK))/(1+10^(pHcyt-pK));

p.Vcytprop=0.4;
[GA4_ave_Vcp04,GPS1_Vcp04]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps)
p.Vcytprop=0.5;
[GA4_ave_Vcp05,GPS1_Vcp05]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);
p.Vcytprop=0.6;
[GA4_ave_Vcp06,GPS1_Vcp06]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);
p.Vcytprop=0.7;
[GA4_ave_Vcp07,GPS1_Vcp07]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);
p.Vcytprop=0.8;
[GA4_ave_Vcp08,GPS1_Vcp08]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps);

figure
hold on
plot(x_ave, GA4_ave_Vcp04,'LineWidth',2)
plot(x_ave, GA4_ave_Vcp05,'LineWidth',2) 
plot(x_ave, GA4_ave_Vcp06,'LineWidth',2) 
plot(x_ave, GA4_ave_Vcp07,'LineWidth',2) 
plot(x_ave, GA4_ave_Vcp08,'LineWidth',2) 
xlim([0 600])
legend1=legend('$\gamma=0.4$','$\gamma=0.5$','$\gamma=0.6$','$\gamma=0.7$','$\gamma=0.8$');
set(legend1,'Interpreter','latex','FontSize',14);
xlabel('Distance from QC ($\mu$m)','Interpreter','latex','FontSize',14)
ylabel(' $\mu$M GA4','Interpreter','latex','FontSize',14)

save simulationdata_affectofparameters_n10_deg01.mat

end

function [GA4_ave,GPS1]=averagereplicates(p,a,Bmax,h,Kd,x_ave,Nreps)

% This simulates the model Nreps times and calculates the average predicted
% GA concentrations.
GA_reps=zeros(Nreps,6001);
for i=1:Nreps
    [GA_data_wt,pos_data_wt,NcellmeriandEZ_wt]=Cellbasedmodel_GA_Arab_prodHillfunc(p);
    GA4=GA_data_wt(end,1:NcellmeriandEZ_wt);
    pos=pos_data_wt(end,1:NcellmeriandEZ_wt);
    GA_reps(i,:)=interp1(pos,GA4,x_ave,'linear');
end

GA4_ave=mean(GA_reps);
GPS1=a+Bmax*GA4_ave.^h./(Kd.^h+GA4_ave.^h);

end

